'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getWebpackErrors;

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _fns = require('../../lib/fns');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let lineSep = `\n  `;
let takeWebpack = ls => lineSep + _fns._.take(ls, 2).join(lineSep);
let split = s => s.split("\n");
let join = s => s.join("\n");

function cleanPath(str) {
  return str.trim().replace(/\.\/\.motion(\/\.internal)?(\/deps(\/internal)?)?/g, '').replace(new RegExp((0, _opts2.default)('appDir'), 'g'), '').replace(new RegExp('Module not found: ', 'g'), '').replace('in /' + _fns.path.relative((0, _opts2.default)('appDir'), (0, _opts2.default)('deps').dir), '') // webpack remove "in /deps/externals.in.js"
  .replace("/externals.in.js\n", '').replace("/internals.in.js\n", '');
}

function getWebpackErrors(where, err, stats) {
  if (err) return err;

  const jsonStats = stats.toJson({
    source: false
  });

  // check errors
  let errors = jsonStats.errors;

  if (errors.length) {
    // debug output everything
    _fns.log.webpack('webpackErrors', errors.join("\n"));

    let messages = errors.map(split).map(takeWebpack)[0].dim;
    let whereMsg = where == 'externals' ? 'NPM modules' : 'imported local modules';
    let message = cleanPath(`Webpack: ${ whereMsg }`.yellow + `  ${ messages }`).grey;

    let file, line, column;

    // this is ridiculous, but webpack only gives us errors as a big string stack dump
    // with the file at the bottom and line number at the end
    try {
      const lastError = errors[errors.length - 1].split("\n");
      const fileAndLine = cleanPath(lastError[lastError.length - 1]).split(' '); // on last line of webpack error

      if (fileAndLine.length > 2) {
        file = fileAndLine[1];
        line = fileAndLine[2].split(':')[0];
        column = fileAndLine[2].split(':')[1];
      }
    } catch (e) {
      return e;
    }

    // rather than error out and break startup of app from running,
    // we just log directly but continue
    return { message, file, loc: { line, column } };
  }

  // check warnings
  if (jsonStats.warnings.length) {
    print('Webpack warnings: ', jsonStats.warnings[0].split("\n").slice(0, 3).join("\n"));
  }

  _fns.log.webpack('webpack finished');
}
//# sourceMappingURL=getWebpackErrors.js.map