'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = webpackConfig;

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _fns = require('../../lib/fns');

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _cache = require('../../cache');

var _cache2 = _interopRequireDefault(_cache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let runnerRoot = _fns.path.resolve(_fns.path.join(__dirname, '..', '..', '..', '..'));
let runnerModules = _fns.path.join(runnerRoot, 'node_modules');

// copy styles into .motion/static/styles
let fileStyleFolder = 'file?name=assets/styles/[name]-[hash].css';

function webpackConfig(filename) {
  let config = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  const conf = (0, _deepmerge2.default)({
    context: runnerRoot,
    debug: (0, _opts2.default)('debug'),
    output: {
      // app/.motion/.internal/deps
      path: (0, _opts2.default)('deps').dir,
      filename,
      library: `${ (0, _opts2.default)('saneName') }-${ filename.replace('.js', '') }`,
      libraryTarget: 'commonjs'
    },
    // from motion-client
    externals: {
      react: 'exports.React',
      'react-dom': 'exports.ReactDOM',
      history: 'exports.history',
      radium: 'exports.radium'
    },
    devtool: 'source-map',
    node: {
      global: false,
      Buffer: false,
      setImmediate: false
    },
    resolveLoader: { root: runnerModules },
    resolve: {
      root: (0, _opts2.default)('motionDir'),
      modulesDirectories: ['node_modules'],
      extensions: ['', '.js', '.jsx', '.css'],
      fallback: [runnerModules]
    },

    // path.join(motionRoot, '..') // fallback to app root for import css
    module: {
      loaders: [{ test: /\.(ttf|eot|woff|svg)$/, loader: 'file?name=assets/files/[name]-[hash].[ext]' }, { test: /\.json$/, loader: 'json' }, {
        test: /\.css$/,
        loaders: [fileStyleFolder, 'css?module&importLoaders=1']
      }, {
        test: /\.(png|jpg|gif)$/,
        loader: 'url?limit=8192&name=[name]-[hash].[ext]'
      }]
    }
  }, config);

  return conf;
}
//# sourceMappingURL=webpackConfig.js.map