'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.file = file;
exports.app = app;
exports.getBabelConfig = getBabelConfig;

var _motionTransform = require('motion-transform');

var _motionTransform2 = _interopRequireDefault(_motionTransform);

var _helpers = require('./helpers');

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _onMeta = require('./onMeta');

var _onMeta2 = _interopRequireDefault(_onMeta);

var _fns = require('../../lib/fns');

var _writeStyle = require('../../lib/writeStyle');

var _writeStyle2 = _interopRequireDefault(_writeStyle);

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let motion = null;
const id = () => {};

// allow these to change per-call
let _onImports, _onExports;
let _onImports2 = _ => _onImports(_);
let _onExports2 = _ => _onExports(_);

function file(config) {
  _onImports = config.onImports;
  _onExports = config.onExports;

  // only instantiate once
  if (!motion) {
    const motionOpts = {
      basePath: (0, _opts2.default)('appDir'),
      production: (0, _helpers.isProduction)(),
      selectorPrefix: (0, _opts2.default)('config').selectorPrefix || '#_motionapp ',
      routing: (0, _opts2.default)('config').routing,
      log: _fns.log,
      onMeta: _onMeta2.default,
      writeStyle: _writeStyle2.default,
      onImports: _ => _onImports2(_),
      onExports: _ => _onExports2(_)
    };

    motion = _motionTransform2.default.file(motionOpts);
  }

  return getBabelConfig({
    plugins: [motion]
  });
}

function app() {
  return {
    whitelist: [],
    retainLines: true,
    comments: true,
    plugins: [_motionTransform2.default.app({ name: (0, _opts2.default)('saneName') })],
    compact: true,
    extra: { production: (0, _helpers.isProduction)() }
  };
}

function getBabelConfig(_ref) {
  let plugins = _ref.plugins;

  const babelConf = {
    breakConfig: true, // avoid reading .babelrc
    jsxPragma: 'view.el',
    stage: 1,
    blacklist: ['es6.tailCall', 'strict'],
    retainLines: (0, _opts2.default)('config').pretty ? false : true,
    comments: true,
    optional: ['regenerator', 'runtime'],
    plugins,
    extra: {
      production: (0, _helpers.isProduction)()
    }
  };

  const userConf = (0, _opts2.default)('config').babel;

  if (userConf) return (0, _deepmerge2.default)(babelConf, userConf);else return babelConf;
}

exports.default = {
  app,
  file
};
//# sourceMappingURL=config.js.map