'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fns = require('./fns');

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

// uses a simple lock to ensure reads and writes are done safely

exports.default = function () {
  var ref = _asyncToGenerator(function* (filePath, _ref) {

    // public

    let read = function () {
      var ref = _asyncToGenerator(function* () {
        try {
          if (cache) return cache;
          logw('reading file', filePath);
          let state = yield (0, _fns.readFile)(filePath);

          cacheStr = state;

          if (json) {
            state = JSON.parse(state);
            cache = state;
          }

          return state;
        } catch (e) {
          logw(e, e.stack);
          return defaultValue;
        }
      });

      return function read() {
        return ref.apply(this, arguments);
      };
    }();

    let write = function () {
      var ref = _asyncToGenerator(function* (writer) {
        try {
          // logw('waiting...')
          if (lock) yield lock;
          const unlock = getLock();
          const state = yield read();
          let result = yield stateWriter(writer, state, unlock);
        } catch (e) {
          (0, _fns.handleError)(e);
        }
      });

      return function write(_x3) {
        return ref.apply(this, arguments);
      };
    }();

    var _ref$debug = _ref.debug;
    let debug = _ref$debug === undefined ? '' : _ref$debug;
    var _ref$json = _ref.json;
    let json = _ref$json === undefined ? false : _ref$json;
    var _ref$defaultValue = _ref.defaultValue;
    let defaultValue = _ref$defaultValue === undefined ? '' : _ref$defaultValue;

    // helpers
    const logw = _fns.log.writer.bind(null, debug.yellow);

    // internal state
    let cache = null;
    let cacheStr = null;
    let isChanged = true;

    // init
    try {
      yield read();
    } catch (e) {
      try {
        yield write(function (_, write) {
          return write(defaultValue);
        });
      } catch (e) {
        (0, _fns.handleError)(e);
      }
    }

    function hasChanged() {
      let result = isChanged;
      logw('hasChanged?', result);
      isChanged = false;
      return result;
    }

    // private
    let lock = null;
    let unlock = null;

    function getLock() {
      // logw('lock', lock)

      if (lock) return unlock;else {
        // logw('no lock, returning new')
        lock = new Promise(function (res) {
          unlock = function unlock() {
            lock = null;
            res();
          };
        });

        return unlock;
      }
    }

    function stateWriter(writer, state, unlock) {
      // logw('about to call writer...')
      return new Promise(function (res, rej) {
        writer(state, (() => {
          var ref = _asyncToGenerator(function* (toWrite) {
            const finish = function finish() {
              unlock();
              res();
            };

            let toWriteRaw;

            // do this before doing equality checks for cache
            if (json) {
              toWriteRaw = toWrite;
              toWrite = JSON.stringify(toWrite, null, 2);
            }

            if (cacheStr == toWrite) {
              logw('hasnt changed!'.bold);
              finish();
              return;
            }

            try {
              yield (0, _fns.writeFile)(filePath, toWrite);

              isChanged = true;
              cacheStr = toWrite;
              cache = toWriteRaw;

              finish();
            } catch (e) {
              rej(e);
            }
          }),
              _this = this;

          return function (_x4) {
            return ref.apply(_this, arguments);
          };
        })());
      });
    }

    // API
    return {
      read,
      write,
      hasChanged
    };
  });

  return function createWriter(_x, _x2) {
    return ref.apply(this, arguments);
  };
}();
//# sourceMappingURL=createWriter.js.map