'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.run = exports.build = exports.startup = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

let startup = exports.startup = function () {
  var ref = _asyncToGenerator(function* () {
    let options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    if (process.env.MOTION_DEBUG) {
      print('total startup time: ', Date.now() - process.env.startedat);
    }

    if (started) return;
    started = true;

    print();

    // order important!
    yield _opts2.default.init(options);

    _fns.log.setLogging();
    yield _disk2.default.init(); // reads versions and sets up readers/writers
    yield _builder2.default.clear.init(); // ensures internal directories set up
    yield Promise.all([_opts2.default.serialize(), // write out opts to state file
    _cache2.default.init(), _bundler2.default.init()]);

    (0, _watchDeletes2.default)();
  });

  return function startup(_x) {
    return ref.apply(this, arguments);
  };
}();

let gulpScripts = function () {
  var ref = _asyncToGenerator(function* (opts) {
    yield _gulp2.default.init(opts);
    yield _gulp2.default.afterBuild();
  });

  return function gulpScripts(_x3) {
    return ref.apply(this, arguments);
  };
}();

let build = exports.build = function () {
  var ref = _asyncToGenerator(function* () {
    let opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    try {
      yield startup(_extends({}, opts, { build: true }));
      yield Promise.all([_bundler2.default.remakeInstallDir(), _builder2.default.clear.buildDir()]);
      yield Promise.all([_gulp2.default.assets(), gulpScripts({ once: opts.once })]);
      yield _bundler2.default.all();
      yield _builder2.default.build();
      if (opts.once) return;
      print();
      process.exit();
    } catch (e) {
      (0, _fns.handleError)(e);
    }
  });

  return function build(_x4) {
    return ref.apply(this, arguments);
  };
}();

let run = exports.run = function () {
  var ref = _asyncToGenerator(function* (opts) {
    try {
      yield startup(opts);
      if (opts.watch) _gulp2.default.assets();
      yield _server2.default.run();
      yield _bridge2.default.activate();
      activateEditor(_bridge2.default);
      yield gulpScripts();
      _cache2.default.serialize(); // write out cache
      yield _bundler2.default.all();
      if (opts.watch) yield _builder2.default.build();
      _keys2.default.init();
    } catch (e) {
      (0, _fns.handleError)(e);
    }
  });

  return function run(_x6) {
    return ref.apply(this, arguments);
  };
}();

var _bridge = require('./bridge');

var _bridge2 = _interopRequireDefault(_bridge);

var _server = require('./server');

var _server2 = _interopRequireDefault(_server);

var _bundler = require('./bundler');

var _bundler2 = _interopRequireDefault(_bundler);

var _builder = require('./builder');

var _builder2 = _interopRequireDefault(_builder);

var _opts = require('./opts');

var _opts2 = _interopRequireDefault(_opts);

var _disk = require('./disk');

var _disk2 = _interopRequireDefault(_disk);

var _gulp = require('./gulp');

var _gulp2 = _interopRequireDefault(_gulp);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _keys = require('./keys');

var _keys2 = _interopRequireDefault(_keys);

var _watchDeletes = require('./lib/watchDeletes');

var _watchDeletes2 = _interopRequireDefault(_watchDeletes);

var _fns = require('./lib/fns');

var _editor = require('./editor');

var _editor2 = _interopRequireDefault(_editor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

// welcome to motion!

let started = false;

function activateEditor(bridge) {
  const editor = new _editor2.default();
  editor.activate(bridge);
}
//# sourceMappingURL=startup.js.map